// rustup install nightly && rustup default nightly
// rustc overflow.rs && ./overflow 127
// rustc -C debuginfo=0 -C opt-level=3 overflow.rs && ./overflow 127

#![feature(bigint_helper_methods)]

use std::env;
use std::num::Saturating;
use std::num::Wrapping;

fn add_default(n: i8) {
    // Normal rust integer operations: overflow-checked and panic in debug mode
    let a: i8 = n;
    let b: i8 = 1;
    println!("{:?}", a + b);
}

fn add_wrapping(n: i8) {
    // Wrapping integer operations: two's complement behaviour
    let a: Wrapping<i8> = Wrapping(n);
    let b: Wrapping<i8> = Wrapping(1);
    println!("{:?}", a + b);
}

fn add_saturating(n: i8) {
    // Saturating integer operations: stop at the limits
    let a: Saturating<i8> = Saturating(n);
    let b: Saturating<i8> = Saturating(1);
    println!("{:?}", a + b);
}

fn add_checked(n: i8) {
    // Force a checked add, even in release mode
    let a: i8 = n;
    let b: i8 = 1;
    match a.checked_add(b) {
        Some(s) => println!("{:?}", s),
        None => println!("Overflow!"),
    };
}

fn add_carry(n: i8) {
    // Grab the carry bit and use it as we like
    let a: i8 = n;
    let b: i8 = 1;
    let (sum, carry_out) = a.carrying_add(b, false);
    println!("{:?} {:?}", sum, carry_out);
}

fn main() {
    let args: Vec<String> = env::args().collect();
    let n = args[1].parse::<i8>().unwrap();
    add_wrapping(n);
    add_saturating(n);
    add_checked(n);
    add_carry(n);
    add_default(n);
}
